@@ -3,8 +3,6 @@
 import os
 import sys
 
-import parsel
-
 
 # Get the project root dir, which is the parent dir of this
 cwd = os.getcwd()
@@ -15,6 +13,8 @@ project_root = os.path.dirname(cwd)
 # version is used.
 sys.path.insert(0, project_root)
 
+import parsel
+
 
 # -- General configuration ---------------------------------------------
 
@@ -134,6 +134,8 @@ intersphinx_mapping = {
 
 # nitpicky = True  # https://github.com/scrapy/cssselect/pull/110
 nitpick_ignore = [
+    ("py:class", "ExpressionError"),
+    ("py:class", "SelectorSyntaxError"),
     ("py:class", "cssselect.xpath.GenericTranslator"),
     ("py:class", "cssselect.xpath.HTMLTranslator"),
     ("py:class", "cssselect.xpath.XPathExpr"),
