@@ -5,7 +5,7 @@ import unittest
 import pickle
 
 import typing
-from typing import Any
+from typing import cast, Any, Optional, Mapping
 
 from lxml import etree
 from lxml.html import HtmlElement
@@ -15,6 +15,8 @@ from parsel import Selector, SelectorList
 from parsel.selector import (
     CannotRemoveElementWithoutRoot,
     CannotRemoveElementWithoutParent,
+    LXML_SUPPORTS_HUGE_TREE,
+    _NOT_SET,
 )
 
 
@@ -65,7 +67,8 @@ class SelectorTestCase(unittest.TestCase):
         )
 
         self.assertEqual(
-            [x.extract() for x in sel.xpath("//input[@name='a']/@name")], ["a"]
+            [x.extract() for x in sel.xpath("//input[@name='a']/@name")],
+            ["a"],
         )
         self.assertEqual(
             [
@@ -230,7 +233,7 @@ class SelectorTestCase(unittest.TestCase):
         sel = self.sscls(text=body)
 
         representation = (
-            f"<Selector xpath='//input/@name' data='{37 * 'b'}...'>"
+            f"<Selector query='//input/@name' data='{37 * 'b'}...'>"
         )
 
         self.assertEqual(
@@ -241,7 +244,7 @@ class SelectorTestCase(unittest.TestCase):
         body = f"<p><input name='{50 * 'b'}' value='\xa9'/></p>"
 
         representation = (
-            "<Selector xpath='//input[@value=\"©\"]/@value' data='©'>"
+            "<Selector query='//input[@value=\"©\"]/@value' data='©'>"
         )
 
         sel = self.sscls(text=body)
@@ -421,7 +424,7 @@ class SelectorTestCase(unittest.TestCase):
     def test_text_or_root_is_required(self) -> None:
         self.assertRaisesRegex(
             ValueError,
-            "Selector needs either text or root argument",
+            "Selector needs text, body, or root arguments",
             self.sscls,
         )
 
@@ -560,7 +563,8 @@ class SelectorTestCase(unittest.TestCase):
 
         self.assertEqual(
             x.xpath(
-                "//somens:a/text()", namespaces={"somens": "http://scrapy.org"}
+                "//somens:a/text()",
+                namespaces={"somens": "http://scrapy.org"},
             ).extract(),
             ["take this"],
         )
@@ -654,7 +658,8 @@ class SelectorTestCase(unittest.TestCase):
         # "xmlns" is still defined
         self.assertEqual(
             x.xpath(
-                "//xmlns:TestTag/@b:att", namespaces={"b": "http://somens.com"}
+                "//xmlns:TestTag/@b:att",
+                namespaces={"b": "http://somens.com"},
             ).extract()[0],
             "value",
         )
@@ -936,7 +941,8 @@ class SelectorTestCase(unittest.TestCase):
         self.assertEqual(
             len(
                 sel.xpath(
-                    "//f:link", namespaces={"f": "http://www.w3.org/2005/Atom"}
+                    "//f:link",
+                    namespaces={"f": "http://www.w3.org/2005/Atom"},
                 )
             ),
             2,
@@ -1031,7 +1037,7 @@ class SelectorTestCase(unittest.TestCase):
             selectorlist_cls = MySelectorList
 
             def extra_method(self) -> str:
-                return "extra" + self.get()
+                return "extra" + cast(str, self.get())
 
         sel = MySelector(text="<html><div>foo</div></html>")
         self.assertIsInstance(sel.xpath("//div"), MySelectorList)
@@ -1108,7 +1114,7 @@ class SelectorTestCase(unittest.TestCase):
         sel.css("body").drop()
         self.assertEqual(sel.get(), "<html></html>")
 
-    def test_deep_nesting(self):
+    def test_deep_nesting(self) -> None:
         lxml_version = parse_version(etree.__version__)
         lxml_huge_tree_version = parse_version("4.2")
 
@@ -1179,6 +1185,73 @@ class SelectorTestCase(unittest.TestCase):
         self.assertEqual(len(sel.css("span")), nest_level)
         self.assertEqual(len(sel.css("td")), 1)
 
+    def test_invalid_type(self) -> None:
+        with self.assertRaises(ValueError):
+            self.sscls("", type="xhtml")
+
+    def test_default_type(self) -> None:
+        text = "foo"
+        selector = self.sscls(text)
+        self.assertEqual(selector.type, "html")
+
+    def test_json_type(self) -> None:
+        obj = 1
+        selector = self.sscls(str(obj), type="json")
+        self.assertEqual(selector.root, obj)
+        self.assertEqual(selector.type, "json")
+
+    def test_html_root(self) -> None:
+        root = etree.fromstring("<html/>")
+        selector = self.sscls(root=root)
+        self.assertEqual(selector.root, root)
+        self.assertEqual(selector.type, "html")
+
+    def test_json_root(self) -> None:
+        obj = 1
+        selector = self.sscls(root=obj)
+        self.assertEqual(selector.root, obj)
+        self.assertEqual(selector.type, "json")
+
+    def test_json_xpath(self) -> None:
+        obj = 1
+        selector = self.sscls(root=obj)
+        with self.assertRaises(ValueError):
+            selector.xpath("//*")
+
+    def test_json_css(self) -> None:
+        obj = 1
+        selector = self.sscls(root=obj)
+        with self.assertRaises(ValueError):
+            selector.css("*")
+
+    def test_invalid_json(self) -> None:
+        text = "<html/>"
+        selector = self.sscls(text, type="json")
+        self.assertEqual(selector.root, None)
+        self.assertEqual(selector.type, "json")
+
+    def test_text_and_root_warning(self) -> None:
+        with warnings.catch_warnings(record=True) as w:
+            Selector(text="a", root="b")
+            self.assertIn("both text and root", str(w[0].message))
+
+    def test_etree_root_invalid_type(self) -> None:
+        selector = Selector("<html></html>")
+        self.assertRaisesRegex(
+            ValueError,
+            "object as root",
+            Selector,
+            root=selector.root,
+            type="text",
+        )
+        self.assertRaisesRegex(
+            ValueError,
+            "object as root",
+            Selector,
+            root=selector.root,
+            type="json",
+        )
+
 
 class ExsltTestCase(unittest.TestCase):
 
@@ -1336,3 +1409,57 @@ class ExsltTestCase(unittest.TestCase):
         assert el.root.getparent() is not None
         el.drop()
         assert sel.get() == "<a><c/></a>"
+
+
+class SelectorBytesInput(Selector):
+    def __init__(
+        self,
+        text: Optional[str] = None,
+        type: Optional[str] = None,
+        body: bytes = b"",
+        encoding: str = "utf8",
+        namespaces: Optional[Mapping[str, str]] = None,
+        root: Optional[Any] = _NOT_SET,
+        base_url: Optional[str] = None,
+        _expr: Optional[str] = None,
+        huge_tree: bool = LXML_SUPPORTS_HUGE_TREE,
+    ) -> None:
+        if text:
+            body = bytes(text, encoding=encoding)
+            text = None
+        super().__init__(
+            text=text,
+            type=type,
+            body=body,
+            encoding=encoding,
+            namespaces=namespaces,
+            root=root,
+            base_url=base_url,
+            _expr=_expr,
+            huge_tree=huge_tree,
+        )
+
+
+class SelectorTestCaseBytes(SelectorTestCase):
+    sscls = SelectorBytesInput
+
+    def test_representation_slice(self) -> None:
+        pass
+
+    def test_representation_unicode_query(self) -> None:
+        pass
+
+    def test_weakref_slots(self) -> None:
+        pass
+
+    def test_check_text_argument_type(self) -> None:
+        self.assertRaisesRegex(
+            TypeError,
+            "body argument should be of type",
+            self.sscls,
+            body="<html/>",
+        )
+
+
+class ExsltTestCaseBytes(ExsltTestCase):
+    sscls = SelectorBytesInput
